from django.apps import apps
from django.conf import settings
from django.contrib import admin
from django.contrib.auth import views as auth_views
from django.urls import include, path, re_path
from django.views.decorators.csrf import csrf_exempt
from django.views.generic import TemplateView
from django.views.i18n import JavaScriptCatalog

import calendarweek.django
from ckeditor_uploader import views as ckeditor_uploader_views
from graphene_django.views import GraphQLView
from health_check.urls import urlpatterns as health_urls
from maintenance_mode.decorators import force_maintenance_mode_off
from oauth2_provider.views import ConnectDiscoveryInfoView
from rules.contrib.views import permission_required
from two_factor.urls import urlpatterns as tf_urls

from . import views

urlpatterns = [
    path("", TemplateView.as_view(template_name="core/vue_index.html"), name="vue_app"),
    path("manifest.json", views.ManifestView.as_view(), name="manifest"),
    path("sw.js", views.ServiceWorkerView.as_view(), name="service_worker"),
    path(settings.MEDIA_URL.removeprefix("/"), include("titofisto.urls")),
    path("__icons__/", include("dj_iconify.urls")),
    path("graphql/", csrf_exempt(GraphQLView.as_view(graphiql=True)), name="graphql"),
    path("logo", force_maintenance_mode_off(views.LogoView.as_view()), name="logo"),
    path(
        ".well-known/openid-configuration/",
        ConnectDiscoveryInfoView.as_view(),
        name="oidc_configuration",
    ),
    path("oauth/", include("oauth2_provider.urls", namespace="oauth2_provider")),
    path(
        "django/",
        include(
            [
                path("", include("django_prometheus.urls")),
                path("account/login/", views.LoginView.as_view(), name="login"),
                path(
                    "accounts/signup/", views.AccountRegisterView.as_view(), name="account_signup"
                ),
                path("accounts/logout/", auth_views.LogoutView.as_view(), name="logout"),
                path(
                    "accounts/password/change/",
                    views.CustomPasswordChangeView.as_view(),
                    name="account_change_password",
                ),
                path(
                    "accounts/password/reset/",
                    views.CustomPasswordResetView.as_view(),
                    name="account_reset_password",
                ),
                path("accounts/", include("allauth.urls")),
                path(
                    "accounts/social/connections/<int:pk>/delete/",
                    views.SocialAccountDeleteView.as_view(),
                    name="delete_social_account_by_pk",
                ),
                path("vue_dummy/", views.vue_dummy, name="vue_dummy"),
                path("offline/", views.OfflineView.as_view(), name="offline"),
                path(
                    "invitations/send-invite/", views.InvitePerson.as_view(), name="invite_person"
                ),
                path(
                    "invitations/code/enter/",
                    views.EnterInvitationCode.as_view(),
                    name="enter_invitation_code",
                ),
                path(
                    "invitations/code/generate/",
                    views.GenerateInvitationCode.as_view(),
                    name="generate_invitation_code",
                ),
                path(
                    "invitations/disabled/",
                    views.InviteDisabledView.as_view(),
                    name="invite_disabled",
                ),
                path("invitations/", include("invitations.urls")),
                path("status/", views.SystemStatus.as_view(), name="system_status"),
                path("", include(tf_urls)),
                path("account/login/", views.TwoFactorLoginView.as_view()),
                path(
                    "account/two_factor/add/",
                    views.TwoFactorSetupView.as_view(),
                    name="setup_two_factor_auth",
                ),
                path("school_terms/", views.SchoolTermListView.as_view(), name="school_terms"),
                path(
                    "school_terms/create/",
                    views.SchoolTermCreateView.as_view(),
                    name="create_school_term",
                ),
                path(
                    "school_terms/<int:pk>/",
                    views.SchoolTermEditView.as_view(),
                    name="edit_school_term",
                ),
                path("persons/", views.persons, name="persons"),
                path(
                    "person/", TemplateView.as_view(template_name="core/empty.html"), name="person"
                ),
                path("persons/create/", views.CreatePersonView.as_view(), name="create_person"),
                path(
                    "persons/<int:id_>/",
                    TemplateView.as_view(template_name="core/empty.html"),
                    name="person_by_id",
                ),
                path(
                    "persons/<int:pk>/edit/",
                    views.EditPersonView.as_view(),
                    name="edit_person_by_id",
                ),
                path("persons/<int:id_>/delete/", views.delete_person, name="delete_person_by_id"),
                path(
                    "persons/<int:pk>/invite/",
                    views.InvitePersonByID.as_view(),
                    name="invite_person_by_id",
                ),
                path("groups/", views.groups, name="groups"),
                path(
                    "groups/additional_fields/", views.additional_fields, name="additional_fields"
                ),
                path("groups/child_groups/", views.groups_child_groups, name="groups_child_groups"),
                path(
                    "groups/additional_fields/<int:id_>/edit/",
                    views.edit_additional_field,
                    name="edit_additional_field_by_id",
                ),
                path(
                    "groups/additional_fields/create/",
                    views.edit_additional_field,
                    name="create_additional_field",
                ),
                path(
                    "groups/additional_fields/<int:id_>/delete/",
                    views.delete_additional_field,
                    name="delete_additional_field_by_id",
                ),
                path("groups/create/", views.edit_group, name="create_group"),
                path("groups/<int:id_>/", views.group, name="group_by_id"),
                path("groups/<int:id_>/edit/", views.edit_group, name="edit_group_by_id"),
                path("groups/<int:id_>/delete/", views.delete_group, name="delete_group_by_id"),
                path("", views.index, name="index"),
                path("dashboard/edit/", views.EditDashboardView.as_view(), name="edit_dashboard"),
                path("groups/group_types/create", views.edit_group_type, name="create_group_type"),
                path(
                    "groups/group_types/<int:id_>/delete/",
                    views.delete_group_type,
                    name="delete_group_type_by_id",
                ),
                path(
                    "groups/group_types/<int:id_>/edit/",
                    views.edit_group_type,
                    name="edit_group_type_by_id",
                ),
                path("groups/group_types/", views.group_types, name="group_types"),
                path("announcements/", views.announcements, name="announcements"),
                path("announcements/create/", views.announcement_form, name="add_announcement"),
                path(
                    "announcements/edit/<int:id_>/",
                    views.announcement_form,
                    name="edit_announcement",
                ),
                path(
                    "announcements/delete/<int:id_>/",
                    views.delete_announcement,
                    name="delete_announcement",
                ),
                path("search/searchbar/", views.searchbar_snippets, name="searchbar_snippets"),
                path("search/", views.PermissionSearchView.as_view(), name="haystack_search"),
                path("maintenance-mode/", include("maintenance_mode.urls")),
                path("impersonate/", include("impersonate.urls")),
                path(
                    "oauth/applications/",
                    views.OAuth2ListView.as_view(),
                    name="oauth2_applications",
                ),
                path(
                    "oauth/applications/register/",
                    views.OAuth2RegisterView.as_view(),
                    name="register_oauth_application",
                ),
                path(
                    "oauth/applications/<int:pk>/",
                    views.OAuth2DetailView.as_view(),
                    name="oauth2_application",
                ),
                path(
                    "oauth/applications/<int:pk>/delete/",
                    views.OAuth2DeleteView.as_view(),
                    name="delete_oauth2_application",
                ),
                path(
                    "oauth/applications/<int:pk>/edit/",
                    views.OAuth2EditView.as_view(),
                    name="edit_oauth2_application",
                ),
                path(
                    "oauth/authorize/",
                    views.CustomAuthorizationView.as_view(),
                    name="oauth2_provider:authorize",
                ),
                path("__i18n__/", include("django.conf.urls.i18n")),
                path(
                    "ckeditor/upload/",
                    permission_required("core.ckeditor_upload_files_rule")(
                        ckeditor_uploader_views.upload
                    ),
                    name="ckeditor_upload",
                ),
                path(
                    "ckeditor/browse/",
                    permission_required("core.ckeditor_upload_files_rule")(
                        ckeditor_uploader_views.browse
                    ),
                    name="ckeditor_browse",
                ),
                path("select2/", include("django_select2.urls")),
                path(
                    "calendarweek_i18n.js", calendarweek.django.i18n_js, name="calendarweek_i18n_js"
                ),
                path("gettext.js", JavaScriptCatalog.as_view(), name="javascript-catalog"),
                path(
                    "preferences/site/",
                    views.preferences,
                    {"registry_name": "site"},
                    name="preferences_site",
                ),
                path(
                    "preferences/person/",
                    views.preferences,
                    {"registry_name": "person"},
                    name="preferences_person",
                ),
                path(
                    "preferences/group/",
                    views.preferences,
                    {"registry_name": "group"},
                    name="preferences_group",
                ),
                path(
                    "preferences/site/<int:pk>/",
                    views.preferences,
                    {"registry_name": "site"},
                    name="preferences_site",
                ),
                path(
                    "preferences/person/<int:pk>/",
                    views.preferences,
                    {"registry_name": "person"},
                    name="preferences_person",
                ),
                path(
                    "preferences/group/<int:pk>/",
                    views.preferences,
                    {"registry_name": "group"},
                    name="preferences_group",
                ),
                path(
                    "preferences/site/<int:pk>/<str:section>/",
                    views.preferences,
                    {"registry_name": "site"},
                    name="preferences_site",
                ),
                path(
                    "preferences/person/<int:pk>/<str:section>/",
                    views.preferences,
                    {"registry_name": "person"},
                    name="preferences_person",
                ),
                path(
                    "preferences/group/<int:pk>/<str:section>/",
                    views.preferences,
                    {"registry_name": "group"},
                    name="preferences_group",
                ),
                path(
                    "preferences/site/<str:section>/",
                    views.preferences,
                    {"registry_name": "site"},
                    name="preferences_site",
                ),
                path(
                    "preferences/person/<str:section>/",
                    views.preferences,
                    {"registry_name": "person"},
                    name="preferences_person",
                ),
                path(
                    "preferences/group/<str:section>/",
                    views.preferences,
                    {"registry_name": "group"},
                    name="preferences_group",
                ),
                path("health/", include(health_urls)),
                path("health/pdf/", views.TestPDFGenerationView.as_view(), name="test_pdf"),
                path(
                    "data_checks/",
                    views.DataCheckView.as_view(),
                    name="check_data",
                ),
                path(
                    "data_checks/run/",
                    views.RunDataChecks.as_view(),
                    name="data_check_run",
                ),
                path(
                    "data_checks/<int:pk>/<str:solve_option>/",
                    views.SolveDataCheckView.as_view(),
                    name="data_check_solve",
                ),
                path(
                    "dashboard_widgets/",
                    views.DashboardWidgetListView.as_view(),
                    name="dashboard_widgets",
                ),
                path(
                    "dashboard_widgets/<int:pk>/edit/",
                    views.DashboardWidgetEditView.as_view(),
                    name="edit_dashboard_widget",
                ),
                path(
                    "dashboard_widgets/<int:pk>/delete/",
                    views.DashboardWidgetDeleteView.as_view(),
                    name="delete_dashboard_widget",
                ),
                path(
                    "dashboard_widgets/<str:app>/<str:model>/new/",
                    views.DashboardWidgetCreateView.as_view(),
                    name="create_dashboard_widget",
                ),
                path(
                    "dashboard_widgets/default/",
                    views.EditDashboardView.as_view(),
                    {"default": True},
                    name="edit_default_dashboard",
                ),
                path(
                    "permissions/global/user/",
                    views.UserGlobalPermissionsListBaseView.as_view(),
                    name="manage_user_global_permissions",
                ),
                path(
                    "permissions/global/group/",
                    views.GroupGlobalPermissionsListBaseView.as_view(),
                    name="manage_group_global_permissions",
                ),
                path(
                    "permissions/object/user/",
                    views.UserObjectPermissionsListBaseView.as_view(),
                    name="manage_user_object_permissions",
                ),
                path(
                    "permissions/object/group/",
                    views.GroupObjectPermissionsListBaseView.as_view(),
                    name="manage_group_object_permissions",
                ),
                path(
                    "permissions/global/user/<int:pk>/delete/",
                    views.UserGlobalPermissionDeleteView.as_view(),
                    name="delete_user_global_permission",
                ),
                path(
                    "permissions/global/group/<int:pk>/delete/",
                    views.GroupGlobalPermissionDeleteView.as_view(),
                    name="delete_group_global_permission",
                ),
                path(
                    "permissions/object/user/<int:pk>/delete/",
                    views.UserObjectPermissionDeleteView.as_view(),
                    name="delete_user_object_permission",
                ),
                path(
                    "permissions/object/group/<int:pk>/delete/",
                    views.GroupObjectPermissionDeleteView.as_view(),
                    name="delete_group_object_permission",
                ),
                path(
                    "permissions/assign/",
                    views.SelectPermissionForAssignView.as_view(),
                    name="select_permission_for_assign",
                ),
                path(
                    "permissions/<int:pk>/assign/",
                    views.AssignPermissionView.as_view(),
                    name="assign_permission",
                ),
            ]
        ),
    ),
    path("admin/", admin.site.urls),
    path("admin/uwsgi/", include("django_uwsgi.urls")),
]

# Use custom server error handler to get a request object in the template
handler500 = views.server_error

# Add URLs for optional features
if hasattr(settings, "TWILIO_ACCOUNT_SID"):
    from two_factor.gateways.twilio.urls import urlpatterns as tf_twilio_urls  # noqa

    urlpatterns += [path("django/", include(tf_twilio_urls))]

# Automatically mount URLs from all installed AlekSIS apps
for app_config in apps.app_configs.values():
    if not app_config.name.startswith("aleksis.apps."):
        continue

    try:
        urlpatterns.append(
            path(f"django/app/{app_config.label}/", include(f"{app_config.name}.urls"))
        )
    except ModuleNotFoundError:
        # Ignore exception as app just has no URLs
        pass  # noqa

urlpatterns.append(
    re_path(
        r"^(?P<url>.*)/$", TemplateView.as_view(template_name="core/vue_index.html"), name="vue_app"
    )
)



from django.template import Library, loader

register = Library()


@register.simple_tag(takes_context=True)
def include_widget(context, widget) -> dict:
    """Render a template with context from a defined widget."""
    template = loader.get_template(widget.get_template())
    request = context["request"]
    context = widget._get_context_safe(request)

    return template.render(context)



from typing import Union

from django.core.exceptions import PermissionDenied
from django.utils import timezone

import graphene
from graphene_django import DjangoObjectType
from graphene_django.forms.mutation import DjangoModelFormMutation

from ..forms import PersonForm
from ..models import DummyPerson, Person
from ..util.core_helpers import get_site_preferences, is_impersonate
from .base import FieldFileType
from .notification import NotificationType


class PersonPreferencesType(graphene.ObjectType):
    theme_design_mode = graphene.String()

    def resolve_theme_design_mode(parent, info, **kwargs):
        return parent["theme__design"]


class PersonType(DjangoObjectType):
    class Meta:
        model = Person

    full_name = graphene.String()
    username = graphene.String()
    userid = graphene.ID()
    photo = graphene.Field(FieldFileType)
    avatar = graphene.Field(FieldFileType)
    avatar_url = graphene.String()
    avatar_content_url = graphene.String()
    secondary_image_url = graphene.String()

    notifications = graphene.List(NotificationType)
    unread_notifications_count = graphene.Int()

    is_dummy = graphene.Boolean()
    is_impersonate = graphene.Boolean()
    preferences = graphene.Field(PersonPreferencesType)

    can_edit_person = graphene.Boolean()
    can_delete_person = graphene.Boolean()
    can_change_person_preferences = graphene.Boolean()
    can_impersonate_person = graphene.Boolean()
    can_invite_person = graphene.Boolean()

    def resolve_street(root, info, **kwargs):  # noqa
        if info.context.user.has_perm("core.view_address_rule", root):
            return root.street
        return None

    def resolve_housenumber(root, info, **kwargs):  # noqa
        if info.context.user.has_perm("core.view_address_rule", root):
            return root.housenumber
        return None

    def resolve_postal_code(root, info, **kwargs):  # noqa
        if info.context.user.has_perm("core.view_address_rule", root):
            return root.postal_code
        return None

    def resolve_place(root, info, **kwargs):  # noqa
        if info.context.user.has_perm("core.view_address_rule", root):
            return root.place
        return None

    def resolve_phone_number(root, info, **kwargs):  # noqa
        if info.context.user.has_perm("core.view_contact_details_rule", root):
            return root.phone_number
        return None

    def resolve_mobile_number(root, info, **kwargs):  # noqa
        if info.context.user.has_perm("core.view_contact_details_rule", root):
            return root.mobile_number
        return None

    def resolve_email(root, info, **kwargs):  # noqa
        if info.context.user.has_perm("core.view_contact_details_rule", root):
            return root.email
        return None

    def resolve_date_of_birth(root, info, **kwargs):  # noqa
        if info.context.user.has_perm("core.view_personal_details_rule", root):
            return root.date_of_birth
        return None

    def resolve_place_of_birth(root, info, **kwargs):  # noqa
        if info.context.user.has_perm("core.view_personal_details_rule", root):
            return root.place_of_birth
        return None

    def resolve_children(root, info, **kwargs):  # noqa
        if info.context.user.has_perm("core.view_personal_details_rule", root):
            return root.children.all()
        return []

    def resolve_guardians(root, info, **kwargs):  # noqa
        if info.context.user.has_perm("core.view_personal_details_rule", root):
            return root.guardians.all()
        return []

    def resolve_member_of(root, info, **kwargs):  # noqa
        if info.context.user.has_perm("core.view_person_groups_rule", root):
            return root.member_of.all()
        return []

    def resolve_owner_of(root, info, **kwargs):  # noqa
        if info.context.user.has_perm("core.view_person_groups_rule", root):
            return root.owner_of.all()
        return []

    def resolve_username(root, info, **kwargs):  # noqa
        return root.user.username if root.user else None

    def resolve_userid(root, info, **kwargs):  # noqa
        return root.user.id if root.user else None

    def resolve_unread_notifications_count(root, info, **kwargs):  # noqa
        return root.unread_notifications_count

    def resolve_photo(root, info, **kwargs):
        if info.context.user.has_perm("core.view_photo_rule", root):
            return root.photo
        return None

    def resolve_avatar(root, info, **kwargs):
        if info.context.user.has_perm("core.view_avatar_rule", root):
            return root.avatar
        return None

    def resolve_avatar_url(root, info, **kwargs):
        if info.context.user.has_perm("core.view_avatar_rule", root) and root.avatar:
            return root.avatar.url
        return root.identicon_url

    def resolve_avatar_content_url(root, info, **kwargs):  # noqa
        # Returns the url for the main image for a person, either the avatar, photo or identicon,
        # based on permissions and preferences
        if get_site_preferences()["account__person_prefer_photo"]:
            if info.context.user.has_perm("core.view_photo_rule", root) and root.photo:
                return root.photo.url
            elif info.context.user.has_perm("core.view_avatar_rule", root) and root.avatar:
                return root.avatar.url

        else:
            if info.context.user.has_perm("core.view_avatar_rule", root) and root.avatar:
                return root.avatar.url
            elif info.context.user.has_perm("core.view_photo_rule", root) and root.photo:
                return root.photo.url

        return root.identicon_url

    def resolve_secondary_image_url(root, info, **kwargs):  # noqa
        # returns either the photo url or the avatar url,
        # depending on the one returned by avatar_content_url

        if get_site_preferences()["account__person_prefer_photo"]:
            if info.context.user.has_perm("core.view_avatar_rule", root) and root.avatar:
                return root.avatar.url
        elif info.context.user.has_perm("core.view_photo_rule", root) and root.photo:
            return root.photo.url
        return None

    def resolve_is_dummy(root: Union[Person, DummyPerson], info, **kwargs):
        return root.is_dummy if hasattr(root, "is_dummy") else False

    def resolve_is_impersonate(root: Person, info, **kwargs):
        return is_impersonate(info.context)

    def resolve_notifications(root: Person, info, **kwargs):
        return root.notifications.filter(send_at__lte=timezone.now()).order_by("read", "-created")

    def resolve_can_edit_person(root, info, **kwargs):  # noqa
        return info.context.user.has_perm("core.edit_person_rule", root)

    def resolve_can_delete_person(root, info, **kwargs):  # noqa
        return info.context.user.has_perm("core.delete_person_rule", root)

    def resolve_can_change_person_preferences(root, info, **kwargs):  # noqa
        return info.context.user.has_perm("core.change_person_preferences_rule", root)

    def resolve_can_impersonate_person(root, info, **kwargs):  # noqa
        return root.user and info.context.user.has_perm("core.impersonate_rule", root)

    def resolve_can_invite_person(root, info, **kwargs):  # noqa
        return (not root.user) and info.context.user.has_perm("core.can_invite_rule", root)


class PersonMutation(DjangoModelFormMutation):
    person = graphene.Field(PersonType)

    class Meta:
        form_class = PersonForm

    @classmethod
    def perform_mutate(cls, form, info):
        if not form.initial:
            if not info.context.user.has_perm("core.create_person_rule"):
                raise PermissionDenied()
        else:
            if not info.context.user.has_perm("core.edit_person_rule", form.instance):
                raise PermissionDenied()
        return super().perform_mutate(form, info)



# Generated by Django 3.1.7 on 2021-04-08 19:15

from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0011_globalpermissions_options'),
    ]

    operations = [
        migrations.AlterField(
            model_name='announcement',
            name='valid_from',
            field=models.DateTimeField(default=django.utils.timezone.now, verbose_name='Date and time from when to show'),
        ),
    ]



from importlib import metadata
from typing import TYPE_CHECKING, Any, Optional, Sequence

import django.apps
from django.contrib.auth.signals import user_logged_in, user_logged_out
from django.db.models.signals import post_migrate, pre_migrate
from django.http import HttpRequest

from dynamic_preferences.signals import preference_updated
from license_expression import Licensing
from oauthlib.common import Request as OauthlibRequest

from .core_helpers import copyright_years
from .spdx import LICENSES

if TYPE_CHECKING:
    from oauth2_provider.models import AbstractApplication


class AppConfig(django.apps.AppConfig):
    """An extended version of DJango's AppConfig container."""

    default = False
    default_auto_field = "django.db.models.BigAutoField"

    def __init_subclass__(cls):
        super().__init_subclass__()
        cls.default = True

    def ready(self):
        super().ready()

        # Register default listeners
        pre_migrate.connect(self.pre_migrate, sender=self)
        post_migrate.connect(self.post_migrate, sender=self)
        preference_updated.connect(self.preference_updated)
        user_logged_in.connect(self.user_logged_in)
        user_logged_out.connect(self.user_logged_out)

        # Getting an app ready means it should look at its config once
        self.preference_updated(self)

    def get_distribution_name(self):
        """Get distribution name of application package."""
        if hasattr(self, "dist_name"):
            return self.dist_name
        elif self.name.lower().startswith("aleksis.apps."):
            return self.name.lower().replace("aleksis.apps.", "AlekSIS-App-")

        return None

    def get_distribution(self):
        """Get distribution of application package."""
        dist_name = self.get_distribution_name()
        if dist_name:
            try:
                dist = metadata.distribution(dist_name)
            except metadata.PackageNotFoundError:
                return None

            return dist

    def get_name(self):
        """Get name of application package."""
        if hasattr(self, "verbose_name"):
            return self.verbose_name
        else:
            dist_name = self.get_distribution_name()
            if dist_name:
                return dist_name
            return self.name

    def get_version(self):
        """Get version of application package."""
        if hasattr(self, "version"):
            return self.version
        else:
            dist = self.get_distribution()
            if dist:
                return dist.version
            else:
                return "unknown"

    @classmethod
    def get_licence(cls) -> tuple:
        """Get tuple of licence information of application package."""
        # Get string representation of licence in SPDX format
        licence = getattr(cls, "licence", None)

        default_dict = {
            "isDeprecatedLicenseId": False,
            "isFsfLibre": False,
            "isOsiApproved": False,
            "licenseId": "unknown",
            "name": "Unknown Licence",
            "referenceNumber": -1,
            "url": "",
        }
        if licence:
            # Parse licence string into object format
            licensing = Licensing(LICENSES.keys())
            parsed = licensing.parse(licence).simplify()
            readable = parsed.render_as_readable()

            # Collect flags about licence combination (drop to False if any licence is False)
            flags = {
                "isFsfLibre": True,
                "isOsiApproved": True,
            }

            # Fill information dictionaries with missing data
            licence_dicts = []
            for symbol in parsed.symbols:
                # Get licence base information, stripping the "or later" mark
                licence_dict = LICENSES.get(symbol.key.rstrip("+"), None)

                if licence_dict is None:
                    # Fall back to the default dict
                    licence_dict = default_dict
                else:
                    # Add missing licence link to SPDX data
                    licence_id = licence_dict["licenseId"]
                    licence_dict["url"] = f"https://spdx.org/licenses/{licence_id}.html"

                # Drop summed up flags to False if this licence is False
                flags["isFsfLibre"] = flags["isFsfLibre"] and licence_dict["isFsfLibre"]
                flags["isOsiApproved"] = flags["isOsiApproved"] and licence_dict["isOsiApproved"]

                licence_dicts.append(licence_dict)

            return (readable, flags, licence_dicts)
        else:
            # We could not find a valid licence
            return ("Unknown", [default_dict])

    @classmethod
    def get_licence_dict(cls):
        """Get licence information of application package."""
        licence = cls.get_licence()
        return {
            "verbose_name": licence[0],
            "flags": licence[1],
            "licences": licence[2],
        }

    @classmethod
    def get_urls(cls):
        """Get list of URLs for this application package."""
        return getattr(cls, "urls", {})
        # TODO Try getting from distribution if not set

    @classmethod
    def get_urls_dict(cls):
        """Get list of URLs for this application package."""
        urls = cls.get_urls()
        return [{"name": key, "url": value} for key, value in urls.items()]

    @classmethod
    def get_copyright(cls) -> Sequence[tuple[str, str, str]]:
        """Get copyright information tuples for application package."""
        copyrights = getattr(cls, "copyright_info", tuple())

        copyrights_processed = []
        for copyright_info in copyrights:
            copyrights_processed.append(
                (
                    # Sort copyright years and combine year ranges for display
                    copyright_info[0]
                    if isinstance(copyright_info[0], str)
                    else copyright_years(copyright_info[0]),
                    copyright_info[1],
                    copyright_info[2],
                )
            )

        return copyrights_processed
        # TODO Try getting from distribution if not set

    @classmethod
    def get_copyright_dicts(cls):
        """Get copyright information dictionaries for application package."""
        infos = cls.get_copyright()
        return [{"years": info[0], "name": info[1], "email": info[2]} for info in infos]

    def preference_updated(
        self,
        sender: Any,
        section: Optional[str] = None,
        name: Optional[str] = None,
        old_value: Optional[Any] = None,
        new_value: Optional[Any] = None,
        **kwargs,
    ) -> None:
        """Call on every app instance if a dynamic preference changes, and once on startup.

        By default, it does nothing.
        """
        pass

    def pre_migrate(
        self,
        app_config: django.apps.AppConfig,
        verbosity: int,
        interactive: bool,
        using: str,
        plan: list[tuple],
        apps: django.apps.registry.Apps,
        **kwargs,
    ) -> None:
        """Call on every app instance before its models are migrated.

        By default, it does nothing.
        """
        pass

    def post_migrate(
        self,
        app_config: django.apps.AppConfig,
        verbosity: int,
        interactive: bool,
        using: str,
        **kwargs,
    ) -> None:
        """Call on every app instance after its models have been migrated.

        By default, asks all models to do maintenance on their default data.
        """
        self._maintain_default_data()

    def user_logged_in(
        self, sender: type, request: Optional[HttpRequest], user: "User", **kwargs
    ) -> None:
        """Call after a user logged in.

        By default, it does nothing.
        """
        pass

    def user_logged_out(
        self, sender: type, request: Optional[HttpRequest], user: "User", **kwargs
    ) -> None:
        """Call after a user logged out.

        By default, it does nothing.
        """
        pass

    @classmethod
    def get_all_scopes(cls) -> dict[str, str]:
        """Return all OAuth scopes and their descriptions for this app."""
        return {}

    @classmethod
    def get_available_scopes(
        cls,
        application: Optional["AbstractApplication"] = None,
        request: Optional[HttpRequest] = None,
        *args,
        **kwargs,
    ) -> list[str]:
        """Return a list of all OAuth scopes available to the request and application."""
        return list(cls.get_all_scopes().keys())

    @classmethod
    def get_default_scopes(
        cls,
        application: Optional["AbstractApplication"] = None,
        request: Optional[HttpRequest] = None,
        *args,
        **kwargs,
    ) -> list[str]:
        """Return a list of all OAuth scopes to always include for this request and application."""
        return []

    @classmethod
    def get_additional_claims(cls, scopes: list[str], request: OauthlibRequest) -> dict[str, Any]:
        """Get claim data for requested scopes."""
        return {}

    def _maintain_default_data(self):
        from django.contrib.auth.models import Permission
        from django.contrib.contenttypes.models import ContentType

        if not self.models_module:
            # This app does not have any models, so bail out early
            return

        for model in self.get_models():
            if hasattr(model, "maintain_default_data"):
                # Method implemented by each model object; can be left out
                model.maintain_default_data()
            if hasattr(model, "extra_permissions"):
                ct = ContentType.objects.get_for_model(model)
                for perm, verbose_name in model.extra_permissions:
                    Permission.objects.get_or_create(
                        codename=perm,
                        content_type=ct,
                        defaults={"name": verbose_name},
                    )



import warnings

from .vite import Command as ViteCommand


class Command(ViteCommand):
    help = "Create Vite bundles for AlekSIS (legacy command alias)"  # noqa

    def handle(self, *args, **options):
        warnings.warn(
            "webpack_bundle is deprecated and will be removed "
            "in AlekSIS-Core 4.0. Use the new vite command instead.",
            UserWarning,
        )

        super().handle(*args, **options)



from django.core.exceptions import PermissionDenied

import graphene
from graphene_django import DjangoObjectType

from ..models import Notification


class NotificationType(DjangoObjectType):
    class Meta:
        model = Notification


class MarkNotificationReadMutation(graphene.Mutation):
    class Arguments:
        id = graphene.ID()  # noqa

    notification = graphene.Field(NotificationType)

    @classmethod
    def mutate(cls, root, info, id):  # noqa
        notification = Notification.objects.get(pk=id)

        if not info.context.user.has_perm("core.mark_notification_as_read_rule", notification):
            raise PermissionDenied()
        notification.read = True
        notification.save()

        return MarkNotificationReadMutation(notification=notification)



# Generated by Django 3.2.12 on 2022-02-23 18:03

import ckeditor.fields
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0036_additionalfields_helptext_required'),
    ]

    operations = [
        migrations.CreateModel(
            name='StaticContentWidget',
            fields=[
                ('dashboardwidget_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='core.dashboardwidget')),
                ('content', ckeditor.fields.RichTextField(verbose_name='Content')),
            ],
            options={
                'verbose_name': 'Static content widget',
                'verbose_name_plural': 'Static content widgets',
            },
            bases=('core.dashboardwidget',),
        ),
    ]



import graphene
from graphene import ObjectType
from two_factor.plugins.email.utils import mask_email
from two_factor.plugins.phonenumber.utils import (
    backup_phones,
    format_phone_number,
    get_available_phone_methods,
    mask_phone_number,
)
from two_factor.plugins.registry import registry
from two_factor.utils import default_device


class TwoFactorDeviceType(ObjectType):
    persistent_id = graphene.ID()
    name = graphene.String()
    method_code = graphene.String()
    verbose_name = graphene.String()
    confirmed = graphene.Boolean()
    method_verbose_name = graphene.String()

    action = graphene.String()
    verbose_action = graphene.String()

    def get_method(root, info, **kwargs):
        if getattr(root, "method", None):
            return registry.get_method(root.method)
        return registry.method_from_device(root)

    def resolve_action(root, info, **kwargs):
        method = TwoFactorDeviceType.get_method(root, info, **kwargs)
        return method.get_action(root)

    def resolve_verbose_action(root, info, **kwargs):
        method = TwoFactorDeviceType.get_method(root, info, **kwargs)
        return method.get_verbose_action(root)

    def resolve_verbose_name(root, info, **kwargs):
        method = TwoFactorDeviceType.get_method(root, info, **kwargs)
        if method.code in ["sms", "call"]:
            return mask_phone_number(format_phone_number(root.number))
        elif method.code == "email":
            email = root.email or root.user.email
            if email:
                return mask_email(email)

        return method.verbose_name

    def resolve_method_verbose_name(root, info, **kwargs):
        method = TwoFactorDeviceType.get_method(root, info, **kwargs)
        return method.verbose_name

    def resolve_method_code(root, info, **kwargs):
        method = TwoFactorDeviceType.get_method(root, info, **kwargs)

        return method.code


class PhoneTwoFactorDeviceType(TwoFactorDeviceType):
    number = graphene.String


class TwoFactorType(ObjectType):
    activated = graphene.Boolean()
    default_device = graphene.Field(TwoFactorDeviceType)
    backup_phones = graphene.List(PhoneTwoFactorDeviceType)
    other_devices = graphene.List(TwoFactorDeviceType)
    backup_tokens_count = graphene.Int()
    phone_methods_available = graphene.Boolean()

    def resolve_backup_tokens_count(root, info, **kwargs):
        try:
            backup_tokens = root.staticdevice_set.all()[0].token_set.count()
        except Exception:
            backup_tokens = 0
        return backup_tokens

    def resolve_phone_methods_available(root, info, **kwargs):
        return bool(get_available_phone_methods())

    def resolve_default_device(root, info, **kwargs):
        return default_device(root)

    def resolve_activated(root, info, **kwargs):
        return bool(default_device(root))

    def resolve_other_devices(root, info, **kwargs):
        main_device = TwoFactorType.resolve_default_device(root, info, **kwargs)
        other_devices = []
        for method in registry.get_methods():
            other_devices += list(method.get_other_authentication_devices(root, main_device))

        return other_devices

    def resolve_backup_phones(root, info, **kwargs):
        return backup_phones(root)



# Generated by Django 3.2.13 on 2022-04-19 16:50

import django.contrib.postgres.fields
from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0039_personal_ical_url'),
    ]

    operations = [
        migrations.AlterField(
            model_name='oauthapplication',
            name='allowed_scopes',
            field=django.contrib.postgres.fields.ArrayField(base_field=models.CharField(max_length=255), blank=True, null=True, size=None, verbose_name='Allowed scopes that clients can request'),
        ),
    ]



import logging
from typing import Any, Optional

from django.contrib import messages
from django.http import HttpRequest


def add_message(
    request: Optional[HttpRequest], level: int, message: str, **kwargs
) -> Optional[Any]:
    """Add a message.

    Add a message to either Django's message framework, if called from a web request,
    or to the default logger.

    Default to DEBUG level.
    """
    if request:
        return messages.add_message(request, level, message, **kwargs)
    else:
        return logging.getLogger(__name__).log(level, message)


def debug(request: Optional[HttpRequest], message: str, **kwargs) -> Optional[Any]:
    """Add a debug message.

    Add a message to either Django's message framework, if called from a web request,
    or to the default logger.

    Default to DEBUG level.
    """
    return add_message(request, messages.DEBUG, message, **kwargs)


def info(request: Optional[HttpRequest], message: str, **kwargs) -> Optional[Any]:
    """Add a info message.

    Add a message to either Django's message framework, if called from a web request,
    or to the default logger.

    Default to INFO level.
    """
    return add_message(request, messages.INFO, message, **kwargs)


def success(request: Optional[HttpRequest], message: str, **kwargs) -> Optional[Any]:
    """Add a success message.

    Add a message to either Django's message framework, if called from a web request,
    or to the default logger.

    Default to SUCCESS level.
    """
    return add_message(request, messages.SUCCESS, message, **kwargs)


def warning(request: Optional[HttpRequest], message: str, **kwargs) -> Optional[Any]:
    """Add a warning message.

    Add a message to either Django's message framework, if called from a web request,
    or to the default logger.

    Default to WARNING level.
    """
    return add_message(request, messages.WARNING, message, **kwargs)


def error(request: Optional[HttpRequest], message: str, **kwargs) -> Optional[Any]:
    """Add an error message.

    Add a message to either Django's message framework, if called from a web request,
    or to the default logger.

    Default to ERROR level.
    """
    return add_message(request, messages.ERROR, message, **kwargs)



# Generated by Django 3.1.4 on 2020-12-21 13:38

import django.contrib.sites.managers
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('sites', '0002_alter_domain_unique'),
        ('core', '0006_dashboard_widget_size'),
    ]

    operations = [
        migrations.CreateModel(
            name='DashboardWidgetOrder',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('extended_data', models.JSONField(default=dict, editable=False)),
                ('order', models.PositiveIntegerField(verbose_name='Order')),
                ('person', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='core.person', verbose_name='Person')),
                ('site', models.ForeignKey(default=1, editable=False, on_delete=django.db.models.deletion.CASCADE, to='sites.site')),
                ('widget', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='core.dashboardwidget', verbose_name='Dashboard widget')),
            ],
            options={
                'verbose_name': 'Dashboard widget order',
                'verbose_name_plural': 'Dashboard widget orders',
            },
            managers=[
            ],
        ),
    ]



# Generated by Django 3.2.8 on 2021-11-05 10:37

import django.contrib.postgres.fields
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0025_oauth_align_user_fk'),
    ]

    operations = [
        migrations.AddField(
            model_name='oauthapplication',
            name='allowed_scopes',
            field=django.contrib.postgres.fields.ArrayField(base_field=models.CharField(max_length=32), blank=True, null=True, size=None, verbose_name='Allowed scopes that clients can request'),
        ),
    ]



# Generated by Django 3.2 on 2021-05-09 10:55

from django.conf import settings
import django.contrib.sites.managers
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('sites', '0002_alter_domain_unique'),
        ('django_celery_results', '0008_chordcounter'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('core', '0015_oauth_permissions'),
    ]

    operations = [
        migrations.CreateModel(
            name='TaskUserAssignment',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('extended_data', models.JSONField(default=dict, editable=False)),
                ('site', models.ForeignKey(default=1, editable=False, on_delete=django.db.models.deletion.CASCADE, to='sites.site')),
                ('task_result', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='django_celery_results.taskresult', verbose_name='Task result')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL, verbose_name='Task user')),
            ],
            options={
                'verbose_name': 'Task user assignment',
                'verbose_name_plural': 'Task user assignments',
            },
            managers=[
                ('objects', django.contrib.sites.managers.CurrentSiteManager()),
            ],
        ),
    ]



import graphene
from graphene_django import DjangoObjectType

from ..models import PDFFile
from .base import FieldFileType


class PDFFileType(DjangoObjectType):
    file = graphene.Field(FieldFileType)

    class Meta:
        model = PDFFile
        exclude = ["html_file"]



from django.contrib.messages.constants import DEFAULT_TAGS

import graphene
from graphene import ObjectType
from graphene_django import DjangoObjectType

from ..models import TaskUserAssignment


class CeleryProgressMessage(ObjectType):
    message = graphene.String(required=True)
    level = graphene.Int(required=True)
    tag = graphene.String(required=True)

    def resolve_message(root, info, **kwargs):
        return root[1]

    def resolve_level(root, info, **kwargs):
        return root[0]

    def resolve_tag(root, info, **kwargs):
        return DEFAULT_TAGS.get(root[0], "info")


class CeleryProgressAdditionalButtonType(ObjectType):
    title = graphene.String(required=True)
    url = graphene.String(required=True)
    icon = graphene.String()


class CeleryProgressMetaType(DjangoObjectType):
    additional_button = graphene.Field(CeleryProgressAdditionalButtonType, required=False)
    task_id = graphene.String(required=True)

    def resolve_task_id(root, info, **kwargs):
        return root.task_result.task_id

    class Meta:
        model = TaskUserAssignment
        fields = (
            "title",
            "back_url",
            "progress_title",
            "error_message",
            "success_message",
            "redirect_on_success_url",
            "additional_button",
        )

    @classmethod
    def get_queryset(cls, queryset, info, perm="core.view_progress_rule"):
        return super().get_queryset(queryset, info, perm)

    def resolve_additional_button(root, info, **kwargs):
        if not root.additional_button_title or not root.additional_button_url:
            return None
        return {
            "title": root.additional_button_title,
            "url": root.additional_button_url,
            "icon": root.additional_button_icon,
        }


class CeleryProgressProgressType(ObjectType):
    current = graphene.Int()
    total = graphene.Int()
    percent = graphene.Float()


class CeleryProgressType(graphene.ObjectType):
    state = graphene.String()
    complete = graphene.Boolean()
    success = graphene.Boolean()
    progress = graphene.Field(CeleryProgressProgressType)
    messages = graphene.List(CeleryProgressMessage)
    meta = graphene.Field(CeleryProgressMetaType)

    def resolve_messages(root, info, **kwargs):  # noqa
        if root["complete"] and isinstance(root["result"], list):
            return root["result"]
        return root["progress"].get("messages", [])


class CeleryProgressFetchedMutation(graphene.Mutation):
    class Arguments:
        task_id = graphene.String(required=True)

    celery_progress = graphene.Field(CeleryProgressType)

    def mutate(root, info, task_id, **kwargs):
        task = TaskUserAssignment.objects.filter(task_result__task_id=task_id)

        if not info.context.user.has_perm("core.view_progress_rule", task):
            return None
        task.result_fetched = True
        task.save()
        progress = task.get_progress_with_meta()
        return CeleryProgressFetchedMutation(celery_progress=progress)



import graphene
from graphene_django import DjangoObjectType

from ..models import CustomMenu, CustomMenuItem


class CustomMenuItemType(DjangoObjectType):
    class Meta:
        model = CustomMenuItem

    name = graphene.Field(graphene.String)
    url = graphene.Field(graphene.String)
    icon = graphene.Field(graphene.String)


class CustomMenuType(DjangoObjectType):
    class Meta:
        model = CustomMenu

    name = graphene.Field(graphene.String)
    items = graphene.List(CustomMenuItemType)

    def resolve_items(root, info, **kwargs):
        return root.items.all()



from django.conf import settings
from django.test import override_settings
from django.urls import reverse

import ldap
import pytest
from django_auth_ldap.config import LDAPSearch

from aleksis.core.models import UserAdditionalAttributes

pytestmark = pytest.mark.django_db

LDAP_BASE = "dc=example,dc=com"
LDAP_SETTINGS = {
    "AUTH_LDAP_GLOBAL_OPTIONS": {
        ldap.OPT_NETWORK_TIMEOUT: 1,
    },
    "AUTH_LDAP_USER_SEARCH": LDAPSearch(LDAP_BASE, ldap.SCOPE_SUBTREE),
}


def test_index_not_logged_in(client):
    response = client.get("/django/")

    assert response.status_code == 302
    assert response["Location"].startswith(reverse(settings.LOGIN_URL))


def test_logout(client, django_user_model):
    username = "foo"
    password = "bar"

    django_user_model.objects.create_user(username=username, password=password)
    client.login(username=username, password=password)

    response = client.get("/django/", follow=True)
    assert response.status_code == 200

    response = client.get(reverse("logout"), follow=True)

    assert response.status_code == 200
    assert "Please login to see this page." in response.content.decode("utf-8")


@override_settings(
    AUTHENTICATION_BACKENDS=[
        "aleksis.core.util.ldap.LDAPBackend",
        "django.contrib.auth.backends.ModelBackend",
    ],
    AUTH_LDAP_SERVER_URI="ldap://[100::0]",
    AUTH_LDAP_SET_USABLE_PASSWORD=True,
    **LDAP_SETTINGS
)
def test_login_ldap_fail_if_previously_ldap_authenticated(client, django_user_model):
    username = "foo"
    password = "bar"

    django_user_model.objects.create_user(username=username, password=password)

    # Logging in with a fresh account should success
    res = client.login(username=username, password=password)
    assert res
    client.get(reverse("logout"), follow=True)

    # Logging in with a previously LDAP-authenticated account should fail
    UserAdditionalAttributes.set_user_attribute(username, "ldap_authenticated", True)
    res = client.login(username=username, password=password)
    assert not res

    # Explicitly noting account has not been used with LDAP should succeed
    UserAdditionalAttributes.set_user_attribute(username, "ldap_authenticated", False)
    res = client.login(username=username, password=password)
    assert res



# Generated by Django 3.2.4 on 2021-07-24 13:14

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0019_fix_uniqueness_per_site'),
    ]

    operations = [
        migrations.AlterField(
            model_name='pdffile',
            name='person',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='pdf_files', to='core.person', verbose_name='Owner'),
        ),
    ]



import graphene


class DynamicRouteType(graphene.ObjectType):
    parent_route_name = graphene.String()

    route_path = graphene.String()
    route_name = graphene.String()

    display_account_menu = graphene.Boolean()
    display_sidenav_menu = graphene.Boolean()
    menu_new_tab = graphene.Boolean()

    menu_title = graphene.String()
    menu_icon = graphene.String()

    menu_permission = graphene.String()
    route_permission = graphene.String()



# Generated by Django 3.0.7 on 2020-06-13 14:34

import django.contrib.postgres.fields.jsonb
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('sites', '0002_alter_domain_unique'),
        ('core', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='SchoolTerm',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('extended_data', models.JSONField(default=dict, editable=False)),
                ('name', models.CharField(max_length=255, unique=True, verbose_name='Name')),
                ('date_start', models.DateField(verbose_name='Start date')),
                ('date_end', models.DateField(verbose_name='End date')),
            ],
            options={
                'verbose_name': 'School term',
                'verbose_name_plural': 'School terms',
            },
        ),
        migrations.AlterModelManagers(
            name='group',
            managers=[
            ],
        ),
        migrations.AlterField(
            model_name='group',
            name='name',
            field=models.CharField(max_length=255, verbose_name='Long name'),
        ),
        migrations.AlterField(
            model_name='group',
            name='short_name',
            field=models.CharField(blank=True, max_length=255, null=True, verbose_name='Short name'),
        ),
        migrations.AddField(
            model_name='group',
            name='school_term',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE,
                                    related_name='+', to='core.SchoolTerm', verbose_name='Linked school term'),
        ),
        migrations.AddConstraint(
            model_name='group',
            constraint=models.UniqueConstraint(fields=('school_term', 'name'), name='unique_school_term_name'),
        ),
        migrations.AddConstraint(
            model_name='group',
            constraint=models.UniqueConstraint(fields=('school_term', 'short_name'), name='unique_school_term_short_name'),
        ),
        migrations.AddField(
            model_name='schoolterm',
            name='site',
            field=models.ForeignKey(default=1, editable=False, on_delete=django.db.models.deletion.CASCADE, to='sites.Site'),
        ),
    ]



# Generated by Django 3.2.4 on 2021-07-24 13:14
import os

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0021_drop_persons_accounts_perm'),
        ('favicon', '0004_faviconimg_favicon_size_rel_unique'),
    ]

    def _migrate_favicons(apps, schema_editor):
        FaviconImg = apps.get_model('favicon', "FaviconImg")
        for favicon_img in FaviconImg.objects.all():
            old_name = favicon_img.faviconImage.name
            new_name = os.path.join("public", old_name)
            storage = favicon_img.faviconImage.storage
            if storage.exists(old_name):
                storage.save(new_name, favicon_img.faviconImage.file)
                favicon_img.faviconImage.name = new_name
                favicon_img.save()

    operations = [
        migrations.RunPython(_migrate_favicons)
    ]



# Generated by Django 3.1.4 on 2021-01-04 13:39

from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0008_data_check_result'),
    ]

    operations = [
        migrations.AddField(
            model_name='dashboardwidgetorder',
            name='default',
            field=models.BooleanField(default=False, verbose_name='Part of the default dashboard'),
        ),
        migrations.AlterField(
            model_name='dashboardwidgetorder',
            name='person',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='core.person', verbose_name='Person'),
        ),
        migrations.AlterModelOptions(
            name='dashboardwidget',
            options={'permissions': (('edit_default_dashboard', 'Can edit default dashboard'),),
                     'verbose_name': 'Dashboard Widget', 'verbose_name_plural': 'Dashboard Widgets'},
        ),
    ]



import pytest

from aleksis.core.models import Group, Person

pytestmark = pytest.mark.django_db


def test_child_groups_recursive():
    g_1st_grade = Group.objects.create(name="1st grade")
    g_1a = Group.objects.create(name="1a")
    g_1b = Group.objects.create(name="1b")
    g_2nd_grade = Group.objects.create(name="2nd grade")
    g_2a = Group.objects.create(name="2a")
    g_2b = Group.objects.create(name="2b")
    g_2c = Group.objects.create(name="2c")
    g_2nd_grade_french = Group.objects.create(name="2nd grade French")

    g_1a.parent_groups.set([g_1st_grade])
    g_1b.parent_groups.set([g_1st_grade])
    g_2a.parent_groups.set([g_2nd_grade])
    g_2b.parent_groups.set([g_2nd_grade])
    g_2c.parent_groups.set([g_2nd_grade])
    g_2nd_grade_french.parent_groups.set([g_2b, g_2c])

    assert g_2nd_grade_french in g_2nd_grade.child_groups_recursive
    assert g_2nd_grade_french in g_2b.child_groups_recursive
    assert g_2nd_grade_french in g_2c.child_groups_recursive
    assert g_2nd_grade_french not in g_2a.child_groups_recursive
    assert g_2nd_grade_french not in g_1st_grade.child_groups_recursive


def test_parent_groups_recursive():
    g_1st_grade = Group.objects.create(name="1st grade")
    g_1a = Group.objects.create(name="1a")
    g_1b = Group.objects.create(name="1b")
    g_2nd_grade = Group.objects.create(name="2nd grade")
    g_2a = Group.objects.create(name="2a")
    g_2b = Group.objects.create(name="2b")
    g_2c = Group.objects.create(name="2c")
    g_2nd_grade_french = Group.objects.create(name="2nd grade French")

    g_1a.parent_groups.set([g_1st_grade])
    g_1b.parent_groups.set([g_1st_grade])
    g_2a.parent_groups.set([g_2nd_grade])
    g_2b.parent_groups.set([g_2nd_grade])
    g_2c.parent_groups.set([g_2nd_grade])
    g_2nd_grade_french.parent_groups.set([g_2b, g_2c])

    assert g_1st_grade in g_1a.parent_groups_recursive
    assert g_2nd_grade in g_2a.parent_groups_recursive
    assert g_2nd_grade in g_2nd_grade_french.parent_groups_recursive
    assert g_1st_grade not in g_2nd_grade_french.parent_groups_recursive


def test_members_recursive():
    g_2nd_grade = Group.objects.create(name="2nd grade")
    g_2a = Group.objects.create(name="2a")
    g_2b = Group.objects.create(name="2b")
    g_2c = Group.objects.create(name="2c")
    g_2nd_grade_french = Group.objects.create(name="2nd grade French")

    g_2a.parent_groups.set([g_2nd_grade])
    g_2b.parent_groups.set([g_2nd_grade])
    g_2c.parent_groups.set([g_2nd_grade])
    g_2nd_grade_french.parent_groups.set([g_2b, g_2c])

    p_2a_1 = Person.objects.create(first_name="A", last_name="B")
    p_2a_2 = Person.objects.create(first_name="A", last_name="B")
    p_2b_1 = Person.objects.create(first_name="A", last_name="B")
    p_2b_2 = Person.objects.create(first_name="A", last_name="B")
    p_2c_1 = Person.objects.create(first_name="A", last_name="B")
    p_2c_2 = Person.objects.create(first_name="A", last_name="B")
    p_french_only = Person.objects.create(first_name="A", last_name="B")

    g_2a.members.set([p_2a_1, p_2a_2])
    g_2b.members.set([p_2b_1, p_2b_2])
    g_2c.members.set([p_2c_1, p_2c_2])
    g_2nd_grade_french.members.set([p_2b_1, p_2c_1, p_french_only])

    assert p_2a_1 in g_2nd_grade.members_recursive
    assert p_2a_2 in g_2nd_grade.members_recursive
    assert p_2b_1 in g_2nd_grade.members_recursive
    assert p_2b_2 in g_2nd_grade.members_recursive
    assert p_2c_1 in g_2nd_grade.members_recursive
    assert p_2c_2 in g_2nd_grade.members_recursive
    assert p_french_only in g_2nd_grade.members_recursive
    assert p_french_only in g_2b.members_recursive
    assert p_french_only in g_2c.members_recursive
    assert p_french_only not in g_2a.members_recursive


def test_member_of_recursive():
    g_2nd_grade = Group.objects.create(name="2nd grade")
    g_2a = Group.objects.create(name="2a")
    g_2b = Group.objects.create(name="2b")
    g_2c = Group.objects.create(name="2c")
    g_2nd_grade_french = Group.objects.create(name="2nd grade French")

    g_2a.parent_groups.set([g_2nd_grade])
    g_2b.parent_groups.set([g_2nd_grade])
    g_2c.parent_groups.set([g_2nd_grade])
    g_2nd_grade_french.parent_groups.set([g_2b, g_2c])

    p_2a_1 = Person.objects.create(first_name="A", last_name="B")
    p_2a_2 = Person.objects.create(first_name="A", last_name="B")
    p_2b_1 = Person.objects.create(first_name="A", last_name="B")
    p_2b_2 = Person.objects.create(first_name="A", last_name="B")
    p_2c_1 = Person.objects.create(first_name="A", last_name="B")
    p_2c_2 = Person.objects.create(first_name="A", last_name="B")
    p_french_only = Person.objects.create(first_name="A", last_name="B")

    g_2a.members.set([p_2a_1, p_2a_2])
    g_2b.members.set([p_2b_1, p_2b_2])
    g_2c.members.set([p_2c_1, p_2c_2])
    g_2nd_grade_french.members.set([p_2b_1, p_2c_1, p_french_only])

    assert g_2nd_grade in p_2a_1.member_of_recursive
    assert g_2nd_grade in p_2a_2.member_of_recursive
    assert g_2nd_grade in p_2b_1.member_of_recursive
    assert g_2nd_grade in p_2b_2.member_of_recursive
    assert g_2nd_grade in p_2c_1.member_of_recursive
    assert g_2nd_grade in p_2c_2.member_of_recursive
    assert g_2nd_grade in p_french_only.member_of_recursive
    assert g_2b in p_french_only.member_of_recursive
    assert g_2c in p_french_only.member_of_recursive


def test_owners_recursive():
    g_2nd_grade = Group.objects.create(name="2nd grade")
    g_2a = Group.objects.create(name="2a")
    g_2b = Group.objects.create(name="2b")

    g_2a.parent_groups.set([g_2nd_grade])
    g_2b.parent_groups.set([g_2nd_grade])

    p_1 = Person.objects.create(first_name="A", last_name="B")
    p_2 = Person.objects.create(first_name="A", last_name="B")

    g_2nd_grade.owners.set([p_1])

    assert p_1 in g_2a.owners_recursive
    assert p_1 in g_2b.owners_recursive
    assert p_2 not in g_2a.owners_recursive
    assert p_2 not in g_2b.owners_recursive


def test_owner_of_recursive():
    g_2nd_grade = Group.objects.create(name="2nd grade")
    g_2a = Group.objects.create(name="2a")
    g_2b = Group.objects.create(name="2b")

    g_2a.parent_groups.set([g_2nd_grade])
    g_2b.parent_groups.set([g_2nd_grade])

    p_1 = Person.objects.create(first_name="A", last_name="B")
    p_2 = Person.objects.create(first_name="A", last_name="B")

    g_2nd_grade.owners.set([p_1])

    assert g_2a in p_1.owner_of_recursive.all()
    assert g_2b in p_1.owner_of_recursive.all()
    assert g_2a not in p_2.owner_of_recursive.all()
    assert g_2b not in p_2.owner_of_recursive.all()



from typing import Any, Dict, List, Optional

from django.conf import settings

from templated_email import send_templated_mail

from aleksis.core.util.core_helpers import get_site_preferences, process_custom_context_processors


def send_email(
    template_name: str,
    recipient_list: List[str],
    context: Dict[str, Any],
    from_email: Optional[str] = None,
    **kwargs,
):
    """Send templated email with data from context processors."""
    processed_context = process_custom_context_processors(settings.NON_REQUEST_CONTEXT_PROCESSORS)
    processed_context.update(context)
    if not from_email:
        from_address = get_site_preferences()["mail__address"]
        from_name = get_site_preferences()["general__title"]
        from_email = f"{from_name} <{from_address}>"
    return send_templated_mail(
        template_name=template_name,
        from_email=from_email,
        recipient_list=recipient_list,
        context=processed_context,
        **kwargs,
    )



# Generated by Django 3.2.8 on 2021-11-04 09:52

# Manual migration to replace Django OAuth Toolkit's models with custom models
# This migration is special, because it needs to work correctly both when applied
# during the initial database migration of the whole project, and when updating
# from an older schema version
#
# It strictly has to be applied before Django OAuth Toolkit's initial migration,
# because when the planner introspects existing models, it will stumble upon the swapped
# foreign key relations that now point to our custom models (as configured in settings).
#
# When run during the initial database migration, this simply works by setting run_before.
#
# When run during an update later, Django will refuse to migrate, because run_before will
# lead to an inconsistent migration history. Django OAuth Toolkit's migrations have already
# been run in the past, and we cannot move a new migration to before that.
#
# Therefore, we treat the two cases explicitly:
#  – If tables from the oauth2_provider label exist, we…
#     …assume that its migrations have been run in the past, and we are in an udpate
#     …forcefully drop the migration from the database to convince Django to let our own
#      migration run first
#     …rename the tables to the new app label through raw SQL, passing the original
#      operations that Django OAuth Toolkit would have done to the planner to spoof them
#      in the state
#  – If no tables from the oauth2_provider label exist, we…
#     …assume that we are in the first ever migration of the project
#     …introspect the operations that Django OAuth Toolkit would do, and re-create them for
#      our own app
#  – In both cases, we then let Django OAuth Toolkit do its migrations like normal; they will
#    will be no-ops as we swapped all models

from django.apps import apps as global_apps
from django.conf import settings
from django.db import connection, migrations
from django.utils.module_loading import import_string

MODEL_NAMES = ("Application", "Grant", "AccessToken", "IDToken", "RefreshToken")
_MIGRATION_NAMES = ["0001_initial", "0002_auto_20190406_1805", "0003_auto_20201211_1314", "0004_auto_20200902_2022"]


class Migration(migrations.Migration):
    """Empty dummy migration to satisfy loader in utility functions."""
    # This migration will later be replaced by the real one.
    # We only need a class called Migration here so the loader we use in
    # the functions below does not complain that this migration does not
    # have a Migration class (yet).
    pass

def _get_oauth_migrations():
    """Get all original migrations from Django OAuth Toolkit."""
    loader = migrations.loader.MigrationLoader(connection)
    return [loader.get_migration("oauth2_provider", name) for name in _MIGRATION_NAMES]

def _get_oauth_migration_operations(for_model=None):
    """Get all original operations from Django Oauth Toolkit and re-create them for the new models."""
    operations = []
    for migration in _get_oauth_migrations():
        for old_operation in migration.operations:
            # We need to re-create a new Operation class because Operations
            # are immutably linked to an app
            op_name, args, kwargs = old_operation.deconstruct()
            if not "." in op_name:
                op_name = f"django.db.migrations.{op_name}"
            op_cls = import_string(op_name)
            if "model_name" in kwargs:
                # If we are not interested for this model, skip it
                if for_model is not None and for_model != "oauth" + kwargs["model_name"].lower():
                    continue
                # We are modifying a field. Replace the model name it belongs to
                kwargs["model_name"] = "OAuth" + kwargs["model_name"]
            elif "name" in kwargs:
                # If we are not interested for this model, skip it
                if for_model is not None and for_model != "oauth" + kwargs["name"].lower():
                    continue
                # We are modifying a model. Replace the full name
                kwargs["name"] = "OAuth" + kwargs["name"]
            operations.append(op_cls(*args, **kwargs))

    return operations

def _get_original_models():
    """Get the original models of Django OAuth Toolkit."""
    try:
        return [global_apps.get_model("oauth2_provider", name) for name in MODEL_NAMES]
    except LookupError:
        return []

def _get_new_models():
    """Get the new customised models."""
    return [global_apps.get_model("core", f"OAuth{name}") for name in MODEL_NAMES]

def _original_tables_exist():
    """Check if original Django OAuth Toolkit tables exist."""
    original_tables = set([model._meta.db_table for model in _get_original_models()])
    return bool(set(connection.introspection.table_names()) & original_tables)


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('core', '0022_public_favicon'),
    ]

    run_before = [("oauth2_provider", _MIGRATION_NAMES[0])]

    operations = []
    if _original_tables_exist():
        # If original tables existed, we nned to copy data and remove them afterwards
        for OldModel, NewModel in zip(_get_original_models(), _get_new_models()):
            operations.append(migrations.RunSQL(
                f"ALTER TABLE IF EXISTS {OldModel._meta.db_table} RENAME TO {NewModel._meta.db_table};",
                reverse_sql=f"ALTER TABLE IF EXISTS {NewModel._meta.db_table} RENAME TO {OldModel._meta.db_table};",
                # We copy the original migrations here to trick the planner into believing the migrations had run normally
                state_operations=_get_oauth_migration_operations(NewModel._meta.model_name)
            ))
    else:
        # We need to copy Django OAuth Toolkit's migrations here to re-create
        # all models, because we cannot auto-generate them due to the big swappable
        # model circular dependency rabbit hole
        operations += _get_oauth_migration_operations()


if _original_tables_exist():
    # Fake-unapply migrations so the planner allows run_before
    for name in _MIGRATION_NAMES:
        connection.cursor().execute(f"DELETE FROM django_migrations WHERE app='oauth2_provider' AND name='{name}';")



import graphene
from graphene_django import DjangoObjectType

from ..util.core_helpers import queryset_rules_filter


class RulesObjectType(DjangoObjectType):
    class Meta:
        abstract = True

    @classmethod
    def get_queryset(cls, queryset, info, perm):
        q = super().get_queryset(queryset, info)

        return queryset_rules_filter(info.context, q, perm)


class FieldFileType(graphene.ObjectType):
    url = graphene.String()
    absolute_url = graphene.String()

    def resolve_url(root, info, **kwargs):
        return root.url if root else ""

    def resolve_absolute_url(root, info, **kwargs):
        return info.context.build_absolute_uri(root.url) if root else ""



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['aleksis',
 'aleksis.core',
 'aleksis.core.management',
 'aleksis.core.management.commands',
 'aleksis.core.migrations',
 'aleksis.core.schema',
 'aleksis.core.templatetags',
 'aleksis.core.tests.browser',
 'aleksis.core.tests.models',
 'aleksis.core.tests.regression',
 'aleksis.core.tests.templatetags',
 'aleksis.core.tests.views',
 'aleksis.core.util']

package_data = \
{'': ['*'],
 'aleksis.core': ['frontend/*',
                  'frontend/app/*',
                  'frontend/components/*',
                  'frontend/components/about/*',
                  'frontend/components/app/*',
                  'frontend/components/celery_progress/*',
                  'frontend/components/generic/*',
                  'frontend/components/group/*',
                  'frontend/components/notifications/*',
                  'frontend/components/pdf/*',
                  'frontend/components/person/*',
                  'frontend/components/two_factor/*',
                  'frontend/css/*',
                  'frontend/messages/*',
                  'frontend/mixins/*',
                  'frontend/plugins/*',
                  'locale/ar/LC_MESSAGES/*',
                  'locale/de_DE/LC_MESSAGES/*',
                  'locale/fr/LC_MESSAGES/*',
                  'locale/la/LC_MESSAGES/*',
                  'locale/nb_NO/LC_MESSAGES/*',
                  'locale/ru/LC_MESSAGES/*',
                  'locale/tr_TR/LC_MESSAGES/*',
                  'locale/uk/LC_MESSAGES/*',
                  'static/*',
                  'static/img/*',
                  'static/js/*',
                  'static/public/*',
                  'templates/*',
                  'templates/account/*',
                  'templates/account/email/*',
                  'templates/components/*',
                  'templates/core/*',
                  'templates/core/additional_field/*',
                  'templates/core/announcement/*',
                  'templates/core/dashboard_widget/*',
                  'templates/core/data_check/*',
                  'templates/core/group/*',
                  'templates/core/group_type/*',
                  'templates/core/pages/*',
                  'templates/core/partials/*',
                  'templates/core/perms/*',
                  'templates/core/person/*',
                  'templates/core/school_term/*',
                  'templates/django_tables2/*',
                  'templates/dynamic_preferences/*',
                  'templates/invitations/*',
                  'templates/invitations/forms/*',
                  'templates/invitations/messages/*',
                  'templates/material/*',
                  'templates/material/fields/*',
                  'templates/oauth2_provider/*',
                  'templates/oauth2_provider/application/*',
                  'templates/search/*',
                  'templates/search/indexes/core/*',
                  'templates/sms/*',
                  'templates/socialaccount/*',
                  'templates/socialaccount/snippets/*',
                  'templates/templated_email/*',
                  'templates/two_factor/*',
                  'templates/two_factor/core/*',
                  'templates/two_factor/profile/*']}

install_requires = \
['Celery[django,redis]>=5.2,<6.0',
 'Django>=4.1,<5.0',
 'Whoosh>=2.7.4,<3.0.0',
 'bs4>=0.0.1,<0.0.2',
 'calendarweek>=0.5.0,<0.6.0',
 'celery-haystack-ng>=2.0,<3.0',
 'celery-progress>=0.1.0,<0.2.0',
 'colour>=0.1.5,<0.2.0',
 'customidenticon>=0.1.5,<0.2.0',
 'django-allauth>=0.52.0,<0.53.0',
 'django-any-js>=1.1,<2.0',
 'django-bleach>=3.0.0,<4.0.0',
 'django-cachalot>=2.3.2,<3.0.0',
 'django-cache-memoize>=0.1.6,<0.2.0',
 'django-celery-beat>=2.2.0,<3.0.0',
 'django-celery-email>=3.0.0,<4.0.0',
 'django-celery-results>=2.0.1,<3.0.0',
 'django-ckeditor>=6.0.0,<7.0.0',
 'django-cleanup>=7.0.0,<8.0.0',
 'django-cleavejs>=0.1.0,<0.2.0',
 'django-colorfield>=0.8.0,<0.9.0',
 'django-cte>=1.1.5,<2.0.0',
 'django-dbbackup>=4.0.0,<5.0.0',
 'django-dynamic-preferences>=1.11,<2.0',
 'django-extensions>=3.1.1,<4.0.0',
 'django-favicon-plus-reloaded>=1.1.5,<2.0.0',
 'django-filter>=2.2.0,<3.0.0',
 'django-guardian>=2.2.0,<3.0.0',
 'django-haystack>=3.1,<4.0',
 'django-health-check>=3.12.1,<4.0.0',
 'django-iconify>=0.3,<0.4',
 'django-impersonate>=1.4,<2.0',
 'django-invitations>=2.0.0,<3.0.0',
 'django-ipware>=4.0,<5.0',
 'django-jsonstore>=0.5.0,<0.6.0',
 'django-maintenance-mode>=0.18.0,<0.19.0',
 'django-material>=1.6.0,<2.0.0',
 'django-model-utils>=4.0.0,<5.0.0',
 'django-oauth-toolkit>=2.0.0,<3.0.0',
 'django-phonenumber-field[phonenumbers]>=6.1,<7.0',
 'django-polymorphic>=3.0.0,<4.0.0',
 'django-prometheus>=2.1.0,<3.0.0',
 'django-reversion>=5.0.0,<6.0.0',
 'django-sass-processor==1.2.2',
 'django-tables2>=2.1,<3.0',
 'django-templated-email>=3.0.0,<4.0.0',
 'django-titofisto>=0.2.0,<0.3.0',
 'django-two-factor-auth[call,phonenumbers,sms,webauthn,yubikey]>=1.15.1,<2.0.0',
 'django-uwsgi-ng>=2.0,<3.0',
 'django-vite>=2.0.2,<3.0.0',
 'django-yarnpkg>=6.0,<7.0',
 'django_select2>=8.0,<9.0',
 'django_widget_tweaks>=1.4.5,<2.0.0',
 'djangorestframework>=3.12.4,<4.0.0',
 'dynaconf[ini,toml,yaml]>=3.1,<4.0',
 'graphene-django>=3.0.0,<4.0.0',
 'haystack-redis>=0.0.1,<0.0.2',
 'html2text>=2020.0.0,<2021.0.0',
 'ipython>=8.0.0,<9.0.0',
 'libsass>=0.22.0,<0.23.0',
 'license-expression>=30.0,<31.0',
 'psutil>=5.7.0,<6.0.0',
 'psycopg2>=2.8,<3.0',
 'pycountry>=22.0.0,<23.0.0',
 'python-gnupg>=0.5.0,<0.6.0',
 'rules>=3.0,<4.0',
 'selenium>=4.4.3,<5.0.0']

extras_require = \
{'ldap': ['django-auth-ldap>=4.0,<5.0'],
 's3': ['django-storages>=1.11.1,<2.0.0', 'boto3>=1.17.33,<2.0.0'],
 'sentry': ['sentry-sdk>=1.4.3,<2.0.0']}

entry_points = \
{'console_scripts': ['aleksis-admin = aleksis.core.__main__:aleksis_cmd']}

setup_kwargs = {
    'name': 'aleksis-core',
    'version': '3.0b1',
    'description': 'AlekSIS (School Information System)\u200a—\u200aCore',
    'long_description': 'AlekSIS (School Information System)\u200a—\u200aCore (Core functionality and app framework)\n=================================================================================\n\nThis is the core of the AlekSIS framework and the official distribution\n(see below). It bundles functionality for all apps, and utilities for\ndevelopers and administrators.\n\nIf you are looking for the AlekSIS standard distribution, i.e. the complete\nsoftware product ready for installation and usage, please visit the `AlekSIS®`_\nwebsite or the distribution repository on `EduGit`_.\n\nFeatures\n--------\n\nThe AlekSIS core currently provides the following features:\n\n* For users:\n\n * Authentication via local account, LDAP, or social accounts\n * Two factor authentication via Yubikey, OTP or SMS\n * Configurable dashboard with widgets\n * User-specific preferences\n * Global search\n * Manage announcements\n * Manage groups and types of groups\n * Manage roles and additional, informative fields per group\n * Manage persons\n * Notifications via SMS, email or dashboard\n * PWA with offline caching\n * User registration, password changes and password reset\n * User invitations with invite codes and targeted invites\n\n* For admins\n\n * `aleksis-admin` script to wrap django-admin with pre-configured settings\n * Manage school terms\n * Custom menu entries (e.g. in footer)\n * Automatic backup of database, static and media files\n * OAuth2 and OpenID Connect provider support\n * Serve prometheus metrics\n * System health and data checks\n * Configuration of low-level settings via configuration files\n * System-wide preferenes\n * Creating dashboard widgets for external links/apps\n\n* For developers\n\n * Generic PDF generation with chromium\n * Caching with Redis\n * Django REST framework for apps to use at own discretion\n * Injection of fields, methods, permissions and properties via custom `ExtensibleModel`\n * K8s compatible, read-only Docker image\n * Object-level permissions and rules with `django-guardian` and `django-rules`\n * Query caching with `django-cachalot`\n * uWSGI and Celery via `django-uwsgi` in development\n * Extensible dashbaord widget system\n * Extensible OAuth/OpenID Connect scope and claims system\n\nLicence\n-------\n\n::\n\n  Copyright © 2017, 2018, 2019, 2020, 2021, 2022, 2023 Jonathan Weth <dev@jonathanweth.de>\n  Copyright © 2017, 2018, 2019, 2020 Frank Poetzsch-Heffter <p-h@katharineum.de>\n  Copyright © 2018, 2019, 2020, 2021, 2022, 2023 Hangzhi Yu <yuha@katharineum.de>\n  Copyright © 2018, 2019, 2020, 2021, 2022, 2023 Julian Leucker <leuckeju@katharineum.de>\n  Copyright © 2019, 2020, 2021, 2022, 2023 Dominik George <dominik.george@teckids.org>\n  Copyright © 2019, 2020, 2021, 2022 Tom Teichler <tom.teichler@teckids.org>\n  Copyright © 2019 mirabilos <thorsten.glaser@teckids.org>\n  Copyright © 2021, 2022 magicfelix <felix@felix-zauberer.de>\n  Copyright © 2021 Lloyd Meins <meinsll@katharineum.de>\n  Copyright © 2022 Benedict Suska <benedict.suska@teckids.org>\n  Copyright © 2022 Lukas Weichelt <lukas.weichelt@teckids.org>\n\n  Licenced under the EUPL, version 1.2 or later, by Teckids e.V. (Bonn, Germany).\n\nPlease see the LICENCE.rst file accompanying this distribution for the\nfull licence text or on the `European Union Public Licence`_ website\nhttps://joinup.ec.europa.eu/collection/eupl/guidelines-users-and-developers\n(including all other official language versions).\n\nTrademark\n---------\n\nAlekSIS® is a registered trademark of the AlekSIS open source project, represented\nby Teckids e.V. Please refer to the `trademark policy`_ for hints on using the trademark\nAlekSIS®.\n\n.. _AlekSIS®: https://aleksis.org\n.. _European Union Public Licence: https://eupl.eu/\n.. _EduGit: https://edugit.org/AlekSIS/official/AlekSIS\n.. _trademark policy: https://aleksis.org/pages/about\n',
    'author': 'Dominik George',
    'author_email': 'dominik.george@teckids.org',
    'maintainer': 'Jonathan Weth',
    'maintainer_email': 'dev@jonathanweth.de',
    'url': 'https://aleksis.org/',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



import os

from django.conf import settings
from django.contrib.auth.models import User
from django.test import override_settings
from django.test.selenium import SeleniumTestCase, SeleniumTestCaseBase
from django.urls import reverse

import pytest
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.wait import WebDriverWait

from aleksis.core.models import Person

pytestmark = pytest.mark.django_db

SeleniumTestCaseBase.external_host = os.environ.get("TEST_HOST", "") or None
SeleniumTestCaseBase.browsers = list(
    filter(bool, os.environ.get("TEST_SELENIUM_BROWSERS", "").split(","))
)
SeleniumTestCaseBase.selenium_hub = os.environ.get("TEST_SELENIUM_HUB", "") or None


@pytest.mark.usefixtures("celery_worker")
@override_settings(CELERY_BROKER_URL="memory://localhost//")
class SeleniumTests(SeleniumTestCase):
    serialized_rollback = True

    @classmethod
    def _screenshot(cls, filename):
        screenshot_path = os.environ.get("TEST_SCREENSHOT_PATH", None)
        if screenshot_path:
            os.makedirs(os.path.join(screenshot_path, cls.browser), exist_ok=True)
            return cls.selenium.save_screenshot(
                os.path.join(screenshot_path, cls.browser, filename)
            )
        else:
            return False

    def _login(self, username="admin", password="admin", with_screenshots=False):
        # Navigate to configured login page
        self.selenium.get(self.live_server_url + reverse(settings.LOGIN_URL))
        if with_screenshots:
            self._screenshot("login_default_superuser_blank.png")

        # Find login form input fields and enter defined credentials
        WebDriverWait(self.selenium, 10).until(
            EC.element_to_be_clickable(
                (By.XPATH, '//label[contains(text(), "Username")]/../input'),
            )
        ).send_keys(username)
        WebDriverWait(self.selenium, 10).until(
            EC.element_to_be_clickable(
                (By.XPATH, '//label[contains(text(), "Password")]/../input'),
            )
        ).send_keys(password)
        if with_screenshots:
            self._screenshot("login_default_superuser_filled.png")

        # Submit form by clicking django-two-factor-auth's Next button
        WebDriverWait(self.selenium, 10).until(
            EC.element_to_be_clickable(
                (By.XPATH, '//button[contains(text(), "Login")]'),
            )
        ).click()
        if with_screenshots:
            self._screenshot("login_default_superuser_submitted.png")

    def _create_person(self, username="admin"):
        user = User.objects.get(username=username)
        person = Person.objects.create(user=user, first_name="Jane", last_name="Doe")
        return person

    def test_index(self):
        self.selenium.get(self.live_server_url + "/")
        assert "AlekSIS" in self.selenium.title
        self._screenshot("index.png")

    def test_login_default_superuser(self):
        self._login("admin", "admin", with_screenshots=True)

        # Should redirect away from login page and not put up an alert about wrong credentials
        assert "Please enter a correct username and password." not in self.selenium.page_source

    # Deactivated for now as Selenium test infrastructure needs a complete rethinking
    # because of the new Vue frontend
    # def test_pdf_generation(self):
    #     self._login()
    #     self._create_person()
    #     self.selenium.get(self.live_server_url + reverse("test_pdf"))
    #     el = WebDriverWait(self.selenium, 20).until(lambda d: ".pdf" in self.selenium.current_url)
    #     self._screenshot("pdf.png")



# Generated by Django 3.2.15 on 2022-10-03 18:38

from django.db import migrations, models
import django.utils.timezone
import oauth2_provider.generators
import oauth2_provider.models


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0042_pdffile_empty'),
    ]

    operations = [
        migrations.AddField(
            model_name='taskuserassignment',
            name='additional_button_icon',
            field=models.CharField(blank=True, max_length=255, verbose_name='Additional button icon'),
        ),
        migrations.AddField(
            model_name='taskuserassignment',
            name='additional_button_title',
            field=models.CharField(blank=True, max_length=255, verbose_name='Additional button title'),
        ),
        migrations.AddField(
            model_name='taskuserassignment',
            name='additional_button_url',
            field=models.URLField(blank=True, verbose_name='Additional button URL'),
        ),
        migrations.AddField(
            model_name='taskuserassignment',
            name='back_url',
            field=models.URLField(blank=True, verbose_name='Back URL'),
        ),
        migrations.AddField(
            model_name='taskuserassignment',
            name='error_message',
            field=models.TextField(blank=True, verbose_name='Error message'),
        ),
        migrations.AddField(
            model_name='taskuserassignment',
            name='success_message',
            field=models.TextField(blank=True, verbose_name='Success message'),
        ),
        migrations.AddField(
            model_name='taskuserassignment',
            name='progress_title',
            field=models.CharField(blank=True, max_length=255, verbose_name='Progress title'),
        ),
        migrations.AddField(
            model_name='taskuserassignment',
            name='redirect_on_success_url',
            field=models.URLField(blank=True, verbose_name='Redirect on success URL'),
        ),
        migrations.AddField(
            model_name='taskuserassignment',
            name='title',
            field=models.CharField(default='Data are processed', max_length=255, verbose_name='Title'),
            preserve_default=False,
        ),
    ]



# Generated by Django 3.2.11 on 2022-01-27 18:52

from email.headerregistry import Group
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('sites', '0002_alter_domain_unique'),
        ('core', '0034_invite_permission'),
    ]

    def _migrate_preference_models_to_unique(apps, schema_editor):
        from aleksis.core.models import GroupPreferenceModel, PersonPreferenceModel, SitePreferenceModel

        models = [GroupPreferenceModel, PersonPreferenceModel, SitePreferenceModel]

        for model in models:
            duplicates = {}
            db_alias = schema_editor.connection.alias
            for obj in model.objects.using(db_alias).all():
                key = f"{obj.instance.pk}__{obj.section}__{obj.name}"
                duplicates.setdefault(key, [])
                duplicates[key].append(obj)

            for key, objs in duplicates.items():
                if len(objs) > 1:
                    found = False
                    for obj in objs:
                        if obj.value == obj.preference.default or found:
                            obj.delete()
                        else:
                            found = True


    operations = [
        migrations.RunPython(_migrate_preference_models_to_unique),
        migrations.AlterUniqueTogether(
            name='grouppreferencemodel',
            unique_together={('instance', 'section', 'name')},
        ),
        migrations.AlterUniqueTogether(
            name='personpreferencemodel',
            unique_together={('instance', 'section', 'name')},
        ),
        migrations.AlterUniqueTogether(
            name='sitepreferencemodel',
            unique_together={('instance', 'section', 'name')},
        ),
    ]



# Generated by Django 3.2 on 2021-04-17 18:47

from django.db import migrations, models

def _get_upload_path(instance, filename):  # noqa
    return f"pdfs/{instance.secret}.pdf"


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0013_pdf_file'),
    ]

    operations = [
        migrations.AlterField(
            model_name='pdffile',
            name='file',
            field=models.FileField(blank=True, null=True, upload_to=_get_upload_path, verbose_name='Generated PDF file'),
        ),
    ]
